var net    = require('net')
  , Socket = net.Socket

var portscanner = exports

/**
 * Finds the first port with a status of 'open', implying the port is in use and
 * there is likely a service listening on it.
 *
 * @param {Number} startPort  - Port to begin status check on (inclusive).
 * @param {Number} endPort    - Last port to check status on (inclusive).
 *                              Defaults to 65535.
 * @param {String} host       - Where to scan. Defaults to 'localhost'.
 * @param {Function} callback - function (error, port) { ... }
 *   - {Object|null} error    - Any errors that occurred while port scanning.
 *   - {Number|Boolean} port  - The first open port found. Note, this is the
 *                              first port that returns status as 'open', not
 *                              necessarily the first open port checked. If no
 *                              open port is found, the value is false.
 */
portscanner.findAPortInUse = function(startPort, endPort, host, callback) {
  findAPortWithStatus('open', startPort, endPort, host, callback)
}

/**
 * Finds the first port with a status of 'closed', implying the port is not in
 * use.
 *
 * @param {Number} startPort  - Port to begin status check on (inclusive).
 * @param {Number} endPort    - Last port to check status on (inclusive).
 *                              Defaults to 65535.
 * @param {String} host       - Where to scan. Defaults to 'localhost'.
 * @param {Function} callback - function (error, port) { ... }
 *   - {Object|null} error    - Any errors that occurred while port scanning.
 *   - {Number|Boolean} port  - The first closed port found. Note, this is the
 *                              first port that returns status as 'closed', not
 *                              necessarily the first closed port checked. If no
 *                              closed port is found, the value is false.
 */
portscanner.findAPortNotInUse = function(startPort, endPort, host, callback) {
  findAPortWithStatus('closed', startPort, endPort, host, callback)
}

/**
 * Checks the status of an individual port.
 *
 * @param {Number} port       - Port to check status on.
 * @param {String} host       - Where to scan. Defaults to 'localhost'.
 * @param {Function} callback - function (error, port) { ... }
 *   - {Object|null} error    - Any errors that occurred while port scanning.
 *   - {String} status        - 'open' if the port is in use.
 *                              'closed' if the port is available.
 */
portscanner.checkPortStatus = function(port, host, callback) {
  host = host || 'localhost'
  var socket = new Socket()

  // Socket connection established, port is open
  socket.on('connect', function() {
    socket.end()
    callback(null, 'open')
  })

  // If no response, assume port is not listening
  socket.setTimeout(400)
  socket.on('timeout', function() {
    socket.end()
    callback(null, 'closed')
  })

  // Assuming the port is not open if an error. May need to refine based on
  // exception
  socket.on('error', function(exception) {
    socket.destroy()
    callback(null, 'closed')
  })

  socket.connect(port, host)
}

function findAPortWithStatus(status, startPort, endPort, host, callback) {
  endPort = endPort || 65535
  var foundPort = false
  var numberOfPortsChecked = 0

  var check = function(port) {
    portscanner.checkPortStatus(port, host, function(error, statusOfPort) {
      numberOfPortsChecked++
      // Only callback once
      if (foundPort === false) {
        if (error) {
          foundPort = true
          callback(error)
        }
        else {
          if (statusOfPort === status) {
            foundPort = true
            callback(null, port)
          }
          // All port checks have returned unavailable
          else if (numberOfPortsChecked === (endPort - startPort + 1)) {
            callback(null, false)
          }
        }
      }
    })
  }

  for (var port = startPort; port <= endPort && foundPort === false; port++) {
    check(port)
  }
}

